* {
    font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif;
    font-size: 16px;
}

body, .zebra-stripe > :nth-child(2n+1), .depth-0 {
    background-color: white;
    padding-bottom: 100px;
}

.consent-title {
    font-size: 24px;
    color: #065683;
}

.pinstripe {
    border-top: 2px solid #0075b4;
}

.logo-container {
    padding-top: 10px;
    width: 70%;
    max-width: 700px;
    min-width: 300px;
    height: auto;
    margin: 0 auto;
}

.enterprise-container:after,
.row:after {
    clear: both;
}

.media .media-content, .radio-block .radio,
.radio-block label {
    display: table-cell;
    vertical-align: middle;
}

.enterprise-container {
    min-width: 300px;
    max-width: 700px;
    margin: 0 auto;
    line-height: 1.5;
    font-size: 14px;
    color: #383838;
}

.enterprise-container * {
    font-size: 14px;
}

.enterprise-container a {
    color: #2489c2;
}

.enterprise-container a:hover {
    color: #0b5c8a;
}

.enterprise-logo {
    width: auto;
    height: 100px;
    display: none;
}

[class*='col-'] {
    padding: 0 30px;
}

@media (min-width: 768px) {
    [class*='col-'] {
        float: left;
    }

    [class*='col-'].col-3 {
        width: 30%;
        padding-left: 0;
    }

    [class*='col-'].col-7 {
        width: 70%;
        padding-right: 0;
        padding-left: 35px;
    }

    .border-left {
        border-left: 1px solid #D9D9D9;
    }

    .enterprise-logo {
        display: block;
    }
}

.partnered-text {
    font-size: 16px;
    color: #414141;
    margin-top: 5px;
    margin-bottom: 20px;
}

.partnered-text strong {
    font-size: 1.05em;
}

.course-confirmation-title {
    font-size: 20px;
    color: #2489c2;
    font-weight: normal;
    margin-bottom: 6px;
}

.course-image {
    width: auto;
    height: 50px;
}

.media {
    margin-bottom: 10px;
    display: table;
    width: 100%;
}

.media .thumbnail,
.media .media-content {
    display: table-cell;
    vertical-align: top;
}

.media .thumbnail {
    width: 55px;
    padding-top: 6px;
}

.media .media-content {
    padding-left: 15px;
}

.media .media-content .course-title {
    margin-bottom: 5px;
    font-size: 16px;
    font-weight: bold;
}

.course-detail {
    font-size: 14px;
    margin-bottom: 10px;
}

.course-detail .course-org {
    font-size: 18px;
}

.course-info,
.course-info span {
    font-size: 1em;
}

.course-info .fa,
.course-info span {
    display: inline-block;
    vertical-align: middle;
    margin-right: 5px;
}

.text-underline {
    text-decoration: underline;
}

.caption {
    margin-bottom: 15px;
    font-size: 16px;
}

.radio-block {
    margin-bottom: 10px;
}

.radio-block .radio,
.radio-block label {
    vertical-align: top;
}

.radio-block .radio {
    width: 25px;
    padding-top: 5px;
}

.radio-block label > span {
    display: block;
}

.btn-continue {
    background: #0075B4;
    border: none;
    color: #fff;
    border-radius: 5px;
    margin-top: 10px;
    padding: 10px 15px;
    box-shadow: none;
    font-weight: bold;
}

.btn-continue:hover,
.btn-continue:focus {
    cursor: pointer;
    background: #0b5177;
}

#messages .alert {
    position: relative;
    padding: 10px 10px 10px 35px;
    border: 1px solid transparent;
    box-shadow: none;
    border-radius: 2px;
    margin-bottom: 8px;
}

#messages .alert > .fa {
    position: absolute;
    left: 11px;
    top: 13px;
    font-size: 16px;
}

#messages .alert span {
    display: block;
}

#messages .success {
  background-color: #ecfaec;
  border-color: #b9edb9;
  color: #3c763d;
}

#messages .info {
  background-color: #f2f8fb;
  border-color: #cce3f0;
  color: #31708f;
}

#messages .warning {
  background-color: #fcf8e3;
  border-color: #faebcc;
  color: #8a6d3b;
}

#messages .error {
  background-color: #f2dede;
  border-color: #ebccd1;
  color: #a94442;
}

.modal {
    display: none; /* Hidden by default */
    position: fixed; /* Stay in place */
    z-index: 1; /* Sit on top */
    left: -10%;
    top: -10%;
    width: 120%; /* Full width */
    height: 120%; /* Full height */
    overflow: auto; /* Enable scroll if needed */
    background-color: rgb(0,0,0); /* Fallback color */
    background-color: rgba(0,0,0,0.6); /* Black w/ opacity */
}

.modal-close-button {
    position: absolute;
    top: 0;
    right: 3px;
    color: white;
    font-size: 1.5em !important;
    border: none;
    line-height: normal !important;
    padding: 2px 5px;
    background-color: transparent;
}

.modal-close-button:hover {
    top: -1px;
    right: -1px;
}

.modal-close-button:focus {
    outline: none;
}

.modal-content {
    position: fixed;
    top: 50%;
    left: 50%;
    right: auto;
    bottom: auto;
    transform: translate(-50%, -50%);
    background-color: #f5f5f5;
    margin: auto;
    max-height: 550px;
    max-width: 700px;
    min-width: 500px;
    overflow: hidden;
    width: 80%;
    padding-top: 30px;
}

.modal-content:before {
    content: '';
    height: 30px;
    display: block;
    background: #0075B4;
    position: absolute;
    left: 0;
    top: 0;
    right: 0;
}

h2#modal-header-text {
    font-weight: bold;
    font-size: 18px;
}

.modal-content .course-detail-modal-scrollable {
    max-height: calc(90vh - 30px); /* account for the height of the top bar with the close button */
    overflow: auto;
    padding: 20px;
    width: 100%;
}

.data-consent-checkbox {
    float: left;
    margin-top: 4px;
    margin-right: 10px;
}

.modal-content .modal-header-wrapper {
    display: flex;
}

.modal-content .modal-header-wrapper .image {
    flex-basis: 33%;
}

.modal-content header {
    margin-bottom: 10px;
}

.modal-content header .image {
    display: none;
}

.modal-content header .image img {
    border: 1px solid #6488bb;
    max-width: 210px;
}

.modal-content header .details .organization {
    display: none;
}

.modal-content header .details .organization img {
    height: 25px;
}

.modal-content header .details h1#modal-header-text {
    font-weight: bold;
    margin-bottom: 0;
    text-align: left;
}

.modal-content header .details .short-description {
    font-size: 14px;
    margin-bottom: 10px;
}

.modal-content .body {
    padding-right: 60px;
}

.modal-content .body .details {
    border: 1px solid rgb(0, 117, 180);
    margin-bottom: 20px;
}

.modal-content .body .details ol {
    list-style: none;
    margin: 0;
    padding: 0 10px;
}

.modal-content .body .details li {
    border-bottom: 1px dotted gray;
    padding: 15px 10px;
}

.modal-content .body .details li:last-child {
    border-bottom: none;
}

.modal-content .body .details li:after {
    content: "";
    display: table;
    clear: both;
}

.modal-content .body .details .icon {
    display: none;
    font-weight: bold;
    margin-right: 10px;
    opacity: .6;
}

.modal-content .body .details .title,
.modal-content .body .details .text,
.modal-content .body .details .text .discount {
    font-size: 13px;
}

.modal-content .body .details .detail-container {
    display: flex;
}

.modal-content .body .details .detail-title-container {
    flex-basis: 25%;
}

.modal-content .body .details .detail-value-container {
    flex-shrink: 1;
}

.modal-content .body .details .text strike {
    font-size: 11px;
}

.modal-content .body .details .text .discount {
    color: green;
}

.modal-content .body .overview * {
    font-size: 14px;
}

.modal-content .body .details ol li span {
    font-size: 16px;
}

.modal-content .body .overview .teacher {
    padding-left: 125px;
    position: relative;
    min-height: 117px;
    margin-bottom: 15px;
}

.modal-content .body .overview .teacher h3 {
    font-weight: bold;
    margin-bottom: 10px;
}

.modal-content .body .overview .teacher .teacher-image {
    position: absolute;
    left: 0;
    top: 0;
}

.modal-content .body .overview h1,
.modal-content .body .overview h2 {
    font-size: 15px;
    font-weight: bold;
}

.modal-content .body a {
    color: #0075B4;
    display: inline-block;
    text-decoration: underline;
    border: none;
    background-color:rgba(0,0,0,0);
    padding: 0 !important;
    vertical-align: bottom;
    font-size: 16px;
}
.modal-content .body a {
    font-weight: normal;
}
.modal-content .body a {
    font-weight: normal;
}

@media (min-width: 768px) {
    .modal-content header {
        margin-bottom: 25px;
    }
    .modal-content header .image {
        display: block;
        width: 210px;
        float: left;
        margin-right: 35px;
    }
    .modal-content header .details .organization {
        display: block;
    }
    .modal-content .body .details .icon {
        display: inline;
    }
}


